package dnd;

import dnd.ui.DropPanel;
import dnd.event.PopupAction;
import dnd.event.PopupListener;
import dnd.model.Module;
import dnd.model.PartTableModel;
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Insets;
import java.awt.Toolkit;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.io.IOException;
import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.ListSelectionModel;
import javax.swing.Popup;
import javax.swing.PopupFactory;
import javax.swing.SwingUtilities;
import javax.swing.UIManager;

/**
 *
 * @author l-ectron-x
 */
public class Main implements PopupListener {

    private JComponent popup;
    private Popup popupContainer;
    private DropPanel dropPanel;

    public static void main(String[] args) {
        new Main();
    }

    public Main() {
        JFrame.setDefaultLookAndFeelDecorated(true);
        UIManager.put("swing.boldMetal", false);
        final JFrame f = new JFrame("DnD-Test");
        f.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
        f.add(createNorthPanel(), BorderLayout.NORTH);
        f.add(createDropPanel(), BorderLayout.CENTER);
        f.pack();
        f.setLocationRelativeTo(null);
        SwingUtilities.invokeLater(() -> {
            f.setVisible(true);
        });
    }

    @Override
    public void showPopup() {
        if (popupContainer == null) {
            if (popup == null) {
                popup = createPopup();
            }
            PopupFactory factory = PopupFactory.getSharedInstance();
            popupContainer = factory.getPopup(dropPanel, popup, dropPanel.getLocationOnScreen().x + dropPanel.getWidth() - popup.getPreferredSize().width, dropPanel.getLocationOnScreen().y);
            SwingUtilities.invokeLater(() -> {
                popupContainer.show();
            });
        } else {
            hidePopup();
        }
    }

    @Override
    public void hidePopup() {
        if (popupContainer != null) {
            popupContainer.hide();
            popupContainer = null;
        }
    }

    private JPanel createNorthPanel() {
        JPanel panel = new JPanel(new FlowLayout(FlowLayout.RIGHT));
        panel.setBackground(panel.getBackground().darker());
        JButton button = new JButton();
        try {
            button.setAction(new PopupAction(this));
        } catch (IOException ex) {
            JOptionPane.showMessageDialog(null,
                    ex.getLocalizedMessage(),
                    ex.getClass().getName(),
                    JOptionPane.WARNING_MESSAGE);
        }
        button.setMargin(new Insets(3, 3, 3, 3));
        panel.add(button);
        return panel;
    }

    private JPanel createDropPanel() {
        dropPanel = new DropPanel();
        Dimension size = Toolkit.getDefaultToolkit().getScreenSize();
        dropPanel.setPreferredSize(new Dimension(size.width * 70 / 100, size.height * 70 / 100));
        dropPanel.addComponentListener(new ComponentAdapter() {
            @Override
            public void componentResized(ComponentEvent ce) {
                if (popupContainer != null) {
                    hidePopup();
                    showPopup();
                }
            }
        });
        return dropPanel;
    }

    private JComponent createPopup() {
        final int insets = 4;
        String[] columnNames = {"Symbol", "Code", "Name"};
        Module module = new Module("Bauteile");
        module.createDummyParts();
        int size = module.size();
        JTable table = new JTable(new PartTableModel<>(columnNames, module));
        table.setFillsViewportHeight(true);
        table.setShowGrid(false);
        table.setIntercellSpacing(new Dimension());
        table.setFocusable(false);
        table.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        table.setDragEnabled(true);
        if (size > 0) {
            table.setRowHeight(module.get(0).getIcon().getIconHeight() + insets);
        }

        JPanel panel = new JPanel(new BorderLayout());
        JLabel label = new JLabel(module.getModuleName(), JLabel.CENTER);
        label.setBackground(Color.DARK_GRAY);
        label.setForeground(Color.WHITE);
        label.setBorder(BorderFactory.createEmptyBorder(insets * 2, 0, insets * 2, 0));
        label.setOpaque(true);
        panel.add(label, BorderLayout.NORTH);
        panel.add(new JScrollPane(table));

        return panel;
    }

}
