package dnd.model;

import javax.swing.ImageIcon;
import javax.swing.event.TableModelListener;
import javax.swing.table.TableModel;

/**
 *
 * @author l-ectron-x
 * @param <T>
 */
public class PartTableModel<T extends Module> implements TableModel {

    private final Module module;
    private final String[] columnNames;

    public PartTableModel(String[] columnNames, Module module) {
        this.columnNames = columnNames;
        this.module = module;
    }

    public Part getPart(int rowIndex) {
        return module.get(rowIndex);
    }

    @Override
    public int getRowCount() {
        return module.size();
    }

    @Override
    public int getColumnCount() {
        if (columnNames != null) {
            return columnNames.length;
        }
        return 0;
    }

    @Override
    public String getColumnName(int columnIndex) {
        return columnNames[columnIndex];
    }

    @Override
    public Class<?> getColumnClass(int columnIndex) {
        switch (columnIndex) {
            case 0:
                return ImageIcon.class;
            default:
                return String.class;
        }
    }

    @Override
    public boolean isCellEditable(int rowIndex, int columnIndex) {
        return false;
    }

    @Override
    public Object getValueAt(int rowIndex, int columnIndex) {
        Part part = module.get(rowIndex);

        switch (columnIndex) {
            case 0:
                return part.getIcon();
            case 1:
                return part.getCode();
            case 2:
                return part.getName();
            default:
                return null;
        }
    }

    @Override
    public void setValueAt(Object part, int rowIndex, int columnIndex) {

    }

    @Override
    public void addTableModelListener(TableModelListener tl) {

    }

    @Override
    public void removeTableModelListener(TableModelListener tl) {

    }

}
